@@ -74,19 +74,24 @@ int _gnutls_srp_gx(opaque * text, size_t textsize, opaque ** result, GNUTLS_MPI
 
 
 /****************
- * Choose a random value b and calculate B = (v + g^b) % N.
+ * Choose a random value b and calculate B = (k*v + g^b) % N.
+ * where k == SHA1(N|g)
  * Return: B and if ret_b is not NULL b.
  */
 GNUTLS_MPI _gnutls_calc_srp_B(GNUTLS_MPI * ret_b, GNUTLS_MPI g, GNUTLS_MPI n, GNUTLS_MPI v)
 {
-	GNUTLS_MPI tmpB, tmpV;
-	GNUTLS_MPI b, B;
+	GNUTLS_MPI tmpB = NULL, tmpV = NULL;
+	GNUTLS_MPI b = NULL, B = NULL, k = NULL;
 	int bits;
+	size_t n_size = 0;
 
-	/* calculate:  B = (3v + g^b) % N 
+	/* get the size of n in bytes */
+	_gnutls_mpi_print( NULL, &n_size, n);
+
+	/* calculate:  B = (k*v + g^b) % N 
 	 */
 	bits = _gnutls_mpi_get_nbits(n);
-	b = _gnutls_mpi_new(bits);	/* FIXME: allocate in secure memory */
+	b = _gnutls_mpi_snew(bits);
 	if (b==NULL) {
 		gnutls_assert();
 		return NULL;
@@ -95,34 +100,35 @@ GNUTLS_MPI _gnutls_calc_srp_B(GNUTLS_MPI * ret_b, GNUTLS_MPI g, GNUTLS_MPI n, GN
 	tmpV = _gnutls_mpi_alloc_like(n);
 
 	if (tmpV == NULL) {
-		_gnutls_mpi_release(&b);
-		return NULL;
+		gnutls_assert();
+		goto error;
 	}
 	
 	_gnutls_mpi_randomize(b, bits, GCRY_STRONG_RANDOM);
 
-	tmpB = _gnutls_mpi_new(bits);	/* FIXME: allocate in secure memory */
+	tmpB = _gnutls_mpi_snew(bits);
 	if (tmpB==NULL) {
 		gnutls_assert();
-		_gnutls_mpi_release( &b);
-		_gnutls_mpi_release(&tmpV);
-		return NULL;
+		goto error;
 	}
 
-	B = _gnutls_mpi_new(bits);	/* FIXME: allocate in secure memory */
-	if (tmpB==NULL) {
+	B = _gnutls_mpi_snew(bits);
+	if (B==NULL) {
 		gnutls_assert();
-		_gnutls_mpi_release( &b);
-		_gnutls_mpi_release( &tmpB);
-		_gnutls_mpi_release(&tmpV);
-		return NULL;
+		goto error;
 	}
 
-	_gnutls_mpi_mul_ui(tmpV, v, 3);
+	k = _gnutls_calc_srp_u( n, g, n_size);
+	if (k == NULL) {
+		gnutls_assert();
+		goto error;
+	}
 
+	_gnutls_mpi_mulm(tmpV, k, v, n);
 	_gnutls_mpi_powm(tmpB, g, b, n);
 	_gnutls_mpi_addm(B, tmpV, tmpB, n);
 
+	_gnutls_mpi_release( &k);
 	_gnutls_mpi_release(&tmpB);
 	_gnutls_mpi_release(&tmpV);
 
@@ -132,9 +138,19 @@ GNUTLS_MPI _gnutls_calc_srp_B(GNUTLS_MPI * ret_b, GNUTLS_MPI g, GNUTLS_MPI n, GN
 		_gnutls_mpi_release(&b);
 
 	return B;
+
+error:
+	_gnutls_mpi_release( &b);
+	_gnutls_mpi_release( &B);
+	_gnutls_mpi_release( &k);
+	_gnutls_mpi_release( &tmpB);
+	_gnutls_mpi_release(&tmpV);
+	return NULL;
 }
 
-GNUTLS_MPI _gnutls_calc_srp_u(GNUTLS_MPI A, GNUTLS_MPI B)
+/* This calculates the SHA1(A | B)
+ */
+GNUTLS_MPI _gnutls_calc_srp_u(GNUTLS_MPI A, GNUTLS_MPI B, size_t n_size)
 {
 	size_t b_size, a_size;
 	opaque *holder, hd[MAX_HASH_SIZE];
@@ -146,17 +162,23 @@ GNUTLS_MPI _gnutls_calc_srp_u(GNUTLS_MPI A, GNUTLS_MPI B)
 	_gnutls_mpi_print( NULL, &a_size, A);
 	_gnutls_mpi_print( NULL, &b_size, B);
 
-	holder_size = a_size + b_size;
+	holder_size = n_size + n_size;
 
-	holder = gnutls_alloca(holder_size);
-	if (holder==NULL) return NULL;
+        if (a_size > n_size || b_size > n_size) {
+    		gnutls_assert();
+    		return NULL; /* internal error */
+        }
+
+	holder = gnutls_calloc(1, holder_size);
+	if (holder == NULL)
+		return NULL;
 	
-	_gnutls_mpi_print( holder, &a_size, A);
-	_gnutls_mpi_print( &holder[a_size], &b_size, B);
+	_gnutls_mpi_print(&holder[n_size - a_size], &a_size, A);
+	_gnutls_mpi_print(&holder[n_size + n_size - b_size], &b_size, B);
 
 	td = _gnutls_hash_init(GNUTLS_MAC_SHA);
 	if (td==NULL) {
-		gnutls_afree(holder);
+		gnutls_free(holder);
 		gnutls_assert();
 		return NULL;
 	}
@@ -167,7 +189,7 @@ GNUTLS_MPI _gnutls_calc_srp_u(GNUTLS_MPI A, GNUTLS_MPI B)
 	 */
 	hash_size = 20; /* SHA */
 	ret = _gnutls_mpi_scan( &res, hd, &hash_size);
-	gnutls_afree(holder);
+	gnutls_free(holder);
 
 	if (ret < 0) {
 		gnutls_assert();
@@ -220,7 +242,7 @@ GNUTLS_MPI _gnutls_calc_srp_A(GNUTLS_MPI * a, GNUTLS_MPI g, GNUTLS_MPI n)
 	int bits;
 
 	bits = _gnutls_mpi_get_nbits(n);
-	tmpa = _gnutls_mpi_new(bits);	/* FIXME: allocate in secure memory */
+	tmpa = _gnutls_mpi_snew(bits);
 	if (tmpa==NULL) {
 		gnutls_assert();
 		return NULL;
@@ -228,7 +250,7 @@ GNUTLS_MPI _gnutls_calc_srp_A(GNUTLS_MPI * a, GNUTLS_MPI g, GNUTLS_MPI n)
 	
 	_gnutls_mpi_randomize(tmpa, bits, GCRY_STRONG_RANDOM);
 
-	A = _gnutls_mpi_new(bits);	/* FIXME: allocate in secure memory */
+	A = _gnutls_mpi_snew(bits);
 	if (A==NULL) {
 		gnutls_assert();
 		_gnutls_mpi_release( &tmpa);
@@ -293,7 +315,11 @@ int _gnutls_calc_srp_x(char *username, char *password, opaque * salt,
 GNUTLS_MPI _gnutls_calc_srp_S2(GNUTLS_MPI B, GNUTLS_MPI g, GNUTLS_MPI x, GNUTLS_MPI a, GNUTLS_MPI u, GNUTLS_MPI n)
 {
 	GNUTLS_MPI S=NULL, tmp1=NULL, tmp2=NULL;
-	GNUTLS_MPI tmp4=NULL, tmp3=NULL;
+	GNUTLS_MPI tmp4=NULL, tmp3=NULL, k = NULL;
+	size_t n_size = 0;
+
+	/* get the size of n in bytes */
+	_gnutls_mpi_print( NULL, &n_size, n);
 
 	S = _gnutls_mpi_alloc_like(n);
 	if (S==NULL)
@@ -305,9 +331,15 @@ GNUTLS_MPI _gnutls_calc_srp_S2(GNUTLS_MPI B, GNUTLS_MPI g, GNUTLS_MPI x, GNUTLS_
 	if (tmp1 == NULL || tmp2 == NULL || tmp3 == NULL) {
 		goto freeall;
 	}
+	
+	k = _gnutls_calc_srp_u( n, g, n_size);
+	if (k==NULL) {
+		gnutls_assert();
+		goto freeall;
+	}
 
 	_gnutls_mpi_powm(tmp1, g, x, n); /* g^x */
-	_gnutls_mpi_mul_ui(tmp3, tmp1, 3); /* 3*g^x */
+	_gnutls_mpi_mulm(tmp3, tmp1, k, n); /* k*g^x mod n */
 	_gnutls_mpi_subm(tmp2, B, tmp3, n);
 
 	tmp4 = _gnutls_mpi_alloc_like(n);
@@ -322,6 +354,7 @@ GNUTLS_MPI _gnutls_calc_srp_S2(GNUTLS_MPI B, GNUTLS_MPI g, GNUTLS_MPI x, GNUTLS_
 	_gnutls_mpi_release(&tmp2);
 	_gnutls_mpi_release(&tmp3);
 	_gnutls_mpi_release(&tmp4);
+	_gnutls_mpi_release(&k);
 	
 	return S;
 
@@ -331,6 +364,7 @@ GNUTLS_MPI _gnutls_calc_srp_S2(GNUTLS_MPI B, GNUTLS_MPI g, GNUTLS_MPI x, GNUTLS_
 		_gnutls_mpi_release(&tmp3);
 		_gnutls_mpi_release(&tmp4);
 		_gnutls_mpi_release(&S);
+		_gnutls_mpi_release(&k);
 		return NULL;
 }
 
@@ -561,18 +595,19 @@ void gnutls_srp_server_set_select_function(gnutls_session session,
   *
   * @username contains the actual username. 
   * The @salt, @verifier, @generator and @prime must be filled
-  * in using the gnutls_malloc().
+  * in using the gnutls_malloc(). For convenience @prime and @generator
+  * may also be one of the static parameters defined in extra.h.
   *
   * In case the callback returned a negative number then gnutls will
   * assume that the username does not exist.
   *
-  * In order to prevent allowing an attack to guess valid usernames,
-  * if username does not exist, the g, and n values should be filled in 
-  * using a random user's parameters. In that case the callback should
+  * In order to prevent attackers from guessing valid usernames,
+  * if a user does not exist, g and n values should be filled in 
+  * using a random user's parameters. In that case the callback must
   * return the special value (1).
   *
   * The callback function will only be called once per handshake.
-  * The callback function should return 0 on success.
+  * The callback function should return 0 on success, while
   * -1 indicates an error.
   *
   **/
@@ -699,34 +734,4 @@ opaque digest[20];
 	return 0;
 }
 
-/* FIXME: Functions for backwards compatibility 
- */
-
-void gnutls_srp_free_client_cred( gnutls_srp_client_credentials sc) {
-	gnutls_srp_free_client_credentials( sc);
-}
-
-void gnutls_srp_free_server_cred( gnutls_srp_server_credentials sc) {
-	gnutls_srp_free_server_credentials( sc);
-}
-
-int gnutls_srp_allocate_client_cred( gnutls_srp_client_credentials *sc) {
-	return gnutls_srp_allocate_client_credentials( sc);
-}
-
-int gnutls_srp_allocate_server_cred( gnutls_srp_server_credentials *sc) 
-{
-	return gnutls_srp_allocate_server_credentials( sc);
-}
-
-int gnutls_srp_set_server_cred_file( gnutls_srp_server_credentials res, char *password_file, char * password_conf_file) 
-{
-	return gnutls_srp_set_server_credentials_file( res, password_file, password_conf_file);
-}
-
-int gnutls_srp_set_client_cred( gnutls_srp_client_credentials res, char *username, char * password) 
-{
-	return gnutls_srp_set_client_credentials( res, username, password);
-}
-
 #endif /* ENABLE_SRP */
